/* This algorithm generates random directed acyclic graph
with given number of nodes and the number of edges is
less than the given edge.

The numbering of nodes starts from zero
The first line of the generated graph contains
number of vertex and edges separated by space*/

// compile this file with c++11
// It takes a file name as commandline argument
// then takes two integers
// first is number of vertices
// second is number of edges

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <vector>
#include <string>
#include <set>
typedef unsigned uint;
using namespace std;


int main(int argc, char const *argv[])
{
	if(argc < 2){
		cout << "Error: Commandline argument" << endl;
		exit(0);
	}
	uint edge,vertex;
	cin >> vertex; //number of vertex
	cin >> edge; //number of edge
	uint i,j,k;
	//this array stores the depth of the nodes
	//it means maximum depth the node can have
	vector<uint> arr(vertex);
	srand(time(NULL));

	//randomly assigning depth to each vertex
	for(i=0; i<vertex; ++i){
		j = rand() % vertex;
		arr[i] = j;
	}

	//this set data structure stores edges and ignores 
	//the duplicate edge
	set<string> ss; 
	i = edge;
	string temp;

	while(i>0){
		j = rand() % vertex;
		k = rand() % vertex;
		//if depth of j is less than k
		//then j should be parent
		if(arr[j] < arr[k]){
			temp = to_string(j) + " " + to_string(k);
			--i;
			ss.insert(temp);
		}
		//if depth of k is less than j
		//then k is the parent
		else if(arr[k] < arr[j]){
			temp = to_string(k) + " " + to_string(j);
			--i;
			ss.insert(temp);
		}
	}

	set<string>::iterator begin, end;
	begin = ss.begin();
	end = ss.end();
	uint size = ss.size();
	ofstream out(argv[1], ios::out);
	out << vertex << " " << size << endl;
	//writing data in the file
	while(begin != end){
		out << *begin << endl;
		++begin;
	}
	out.close();
	return 0;
}