//this program takes input graph as comandline argument

//it first creates two graph in form of adjacency list
//one contains the child of each node and other contains
//parent of each node

//for graph with more than 10,000,000 edges change the
//data type of the variable branch in Node class to uint64_t

#include <iostream>
#include <stdint.h>
#include <vector>
#include <fstream>
#include <cstdlib>
using namespace std;

typedef unsigned uint;

//this class is used to populate the linked list
class node
{
	friend class graph;
public:
	explicit node(uint a, node *temp) : data(a){
		next = temp;
	}
	explicit node(uint a) : data(a){
		next = 0;
	}
private:
	node *next; //pointer to next node in the list
	uint data;
};

//this class is used to make an array which contains
//the parent of the nodes
class NodeW{
	friend class graph;
public:
	NodeW() : child(0){
		next = 0;
	}
private:
	//total child nodes, which is actually parent nodes
	uint child;
	node *next;
};

//this class is used to make an array which contains
//the child of the nodes
class Node
{
	friend class graph;
public:
	Node() : child(0), branch(0), visited(false){
		next = 0;
	}
private:
	bool visited; //flag used in different utility function
	//total number of path considering the node as root
	uint branch; 
	uint child; //total child nodes
	node *next;
};

//This class contains all the utility functions required
//for creating a graph and processing it
class graph
{
public:
	//constructor
	explicit graph(uint a, uint b) :
	vertex(a), edge(b){
		//allocate memory to array containing parents of
		//the nodes
		parentarr = new NodeW[a];
		//allocate memory to array containing child of 
		//the nodes
		childarr = new Node[a];
		//low memory, raise error
		if(parentarr == 0){
			cerr << "Error: Memory allocation failed" << endl;
			exit(1);
		}
	}

	//function to add edges
	void setEdge(uint a, uint b){
		temp = new node(b, childarr[a].next);
		childarr[a].next = temp;
		childarr[a].child += 1;
		temp = new node(a, parentarr[b].next);
		parentarr[b].next = temp;
		parentarr[b].child += 1;
	}

	//function to optimize path
	void minimize(uint, uint);

	//destructor
	~graph(){
		node *curr;
		for(uint i=0; i<vertex; ++i){
			curr = childarr[i].next;
			while(curr != 0){
				temp = curr;
				curr = curr->next;
				delete temp;
			}
			curr = parentarr[i].next;
			while(curr != 0){
				temp = curr;
				curr = curr->next;
				delete temp;
			}
		}
		delete [] childarr;
		delete [] parentarr;
	}
	void initialize();

private:
	void change(uint);
	void listmake(uint,vector<uint> &,vector<uint> &);
	void makelist(uint,vector<uint> &,vector<uint> &);
	void makesinglelist(uint, vector<uint> &);
	uint calcBranch(uint);
	bool check(uint, uint);
	void printlist(vector<uint>&);
	void printlist(vector<uint>&,vector<uint>&,vector<uint>&);
	
	//pointer to array containing parent of each node
	NodeW *parentarr; 
	//pointer to array containing child of each node
	Node *childarr;
	node *temp;
	uint vertex;
	uint edge;
};

//this function calculates the total path emerged from
//each node
uint graph::calcBranch(uint index)
{
	if(childarr[index].visited == false){
		childarr[index].visited = true;
		if(childarr[index].child == 0){
			childarr[index].branch = 1;
		}
		else{
			node *curr;
			curr = childarr[index].next;
			uint &branch = childarr[index].branch;
			while(curr!=0){
				branch += calcBranch(curr->data);
				curr = curr->next;
			}
		}
	}
	return childarr[index].branch;
}

//helper function to calculate branch
void graph::initialize()
{
	for(uint i=0; i<vertex; ++i){
		calcBranch(i);
	}
}

//this function with makelist() function, populates main,
//and temp array
void graph::listmake(uint index, vector<uint> &main,
	vector<uint> &temp){
	if(childarr[index].child == 1){
		main.push_back(index);
		node *curr;
		curr = parentarr[index].next;
		while(curr != 0){
			listmake(curr->data, main, temp);
			curr = curr->next;
		}
	}
	else if(childarr[index].child == 2){
		temp.push_back(index);
	}
}

//this function populate main array and temp array of 
//given node
void graph::makelist(uint index,vector<uint> &main,
		vector<uint> &temp){
	main.push_back(index);
	node *curr;
	curr = parentarr[index].next;
	while(curr != 0){
		listmake(curr->data, main, temp);
		curr = curr->next;
	}
}

//this function is used to change the flag of each node
//this function is used after check() function, which
//changes the flag value while processing graph
void graph::change(uint index){
	if(childarr[index].visited == false){
		childarr[index].visited = true;
		node *curr;
		curr = childarr[index].next;
		while(curr != 0){
			change(curr->data);
			curr = curr->next;
		}
	}
}

//this function populate only single list from which
//both the nodes are traceable
void graph::makesinglelist(uint index, vector<uint> &glob){
	if(childarr[index].child == 1){
		glob.push_back(index);
		node *curr;
		curr = parentarr[index].next;
		while(curr != 0){
			makesinglelist(curr->data,glob);
			curr = curr->next;
		}
	}
}


//this function checks if node b is traceable from node a
bool graph::check(uint a, uint b){
	if(childarr[a].branch < childarr[b].branch){
		return false;
	}
	if(a == b){
		return true;
	}
	if(childarr[a].visited == false){
		return false;
	}
	childarr[a].visited = false;
	node *curr;
	curr = childarr[a].next;
	while(curr!=0){
		if(check(curr->data,b)){
			return true;
		}
		curr = curr->next;
	}
	return false;
}

//this function optimizes the path by selecting only
//favourable nodes
void graph::minimize(uint a, uint b)
{
	vector<uint> mainA, mainB, globlist;
	bool value = false;
	//if b is traceable from a
	if(check(a,b)){
		value = true;
	}
	//if a is traceable from b
	else if(check(b,a)){
		uint c = b;
		b = a;
		a = c;
		value = true;
	}

	//this part initialize the flag of those nodes
	//that are changed by check() function
	change(a);
	change(b);

	//if one node is traceable from other, then only
	//make one array which contains the nodes from which
	//both nodes are traceable
	if(value){
		globlist.push_back(a);
		node *curr = parentarr[a].next;
		while(curr != 0){
			makesinglelist(curr->data,globlist);
			curr = curr->next;
		}
		printlist(globlist);
	}
	else{
		vector<uint> tempA, tempB;
		makelist(a,mainA,tempA);
		makelist(b,mainB,tempB);
		uint i,j;
		uint sizeA = tempA.size();
		uint sizeB = tempB.size();
		for(i=0; i<sizeA; ++i){
			for(j=0; j<sizeB; ++j){
				if(tempA[i] == tempB[j]){
					globlist.push_back(tempA[i]);
					node *curr;
					curr = parentarr[tempA[i]].next;
					while(curr != 0){
						makesinglelist(curr->data,globlist);
						curr = curr->next;
					}
				}
			}
		}
		printlist(globlist,mainA,mainB);
	}	  
}

//this function prints the nodes in the case, when one
//node is traceable from other
void graph::printlist(vector<uint> &glob){
	cout << "Just lock single node" << endl;
	uint size = glob.size();
	for( uint i=0; i<size; ++i){
		cout << glob[i] << endl;
	}
	cout << endl;
}

//this function prints all possible combination of nodes
//such that cost is optimized
void graph::printlist(vector<uint> &glob,
	vector<uint> &mainA, vector<uint> &mainB){
	uint size, i, size1, j;
	//this prints the nodes from which both the given
	//nodes are traceable
	if(!glob.empty()){
		cout << "Just lock single node" << endl;
		size = glob.size();
		for(i=0; i<size; ++i){
			cout << glob[i] << endl;
		}
		cout << endl;
	}
	//this part prints the all combination of nodes 
	cout << "Lock the tuples" << endl;
	size1 = mainA.size();
	size = mainB.size();
	for(i=0; i<size1; ++i){
		for(j=0; j<size; ++j){
			cout << "(" << mainA[i] << "," << mainB[j] << ")" <<endl;
		}
	}
}


int main(int argc, char const *argv[])
{
	if(argc < 2){
		cout << "Error: Provide input graph as commandline argument" <<endl;
		exit(1);
	}
	ifstream input(argv[1], ios::in);
	if(!input){
		cerr << "Error: File could not be opened" << endl;
		exit(1);
	}
	uint edge, vertex;
	input >> vertex >> edge;
	graph g(vertex, edge);
	uint u, v;
	//populate graph
	for(uint i=0; i<edge; ++i){
		input >> u >> v;
		if( u!=v){
			g.setEdge(u, v);
		}
	}
	input.close();
	//initialize graph for processing
	g.initialize();
	srand(time(NULL));

	for(int i=0; i<3; ++i){
		u = rand() % vertex;
		v = rand() % vertex;
		cout << "node to be locked" <<endl;
		cout << u << "	" << v << endl;
		g.minimize(u, v);
		cout << endl;
	}
	return 0;
}